@@ -5,7 +5,7 @@ use Carp;
 use Data::Dumper;
 use Scalar::Util 'looks_like_number';
 
-our $VERSION = '0.2.0';
+our $VERSION = '0.2.1';
 
 use base qw/Exporter/;
 our @EXPORT = qw (
@@ -228,7 +228,7 @@ use Inline C => config =>
 
 use Inline C => JavaScript::Duktape::C::libPath::getPath('duktape_wrap.c');
 
-my $SUB = 0;
+
 my $Functions = {};
 
 sub push_perl {
@@ -289,6 +289,7 @@ sub push_perl {
 sub to_perl {
     my $self = shift;
     my $index = shift;
+    my $prev = shift;
     my $ret;
     
     my $type = $self->get_type($index);
@@ -323,9 +324,26 @@ sub to_perl {
         $self->enum($index, JavaScript::Duktape::DUK_ENUM_OWN_PROPERTIES_ONLY);
 
         while ($self->next(-1, 1)) {
-            my $key = $self->to_perl(-2);
-            my $val = $self->to_perl(-1);
+            my ($key, $val);
+
+            #I'm not sure why I need to
+            #substract 4 from stack top!!
+            my $top = $self->get_top() - 4;
             
+            $key = $self->to_perl(-2);
+            my $found = 0;
+            while ($top--){
+                last if $top == -1;
+                if ($self->strict_equals(-1, $top)){
+                    $found = 1;
+                    $val = defined $prev ? $prev : $ret;
+                    last;
+                }
+            }
+
+            ##no circular object found
+            $val = $self->to_perl(-1, $ret) if !$found;
+
             $self->pop_n(2);
 
             if ($isArray) {
@@ -334,6 +352,7 @@ sub to_perl {
                 $ret->{$key} = $val;
             }
         }
+
         $self->pop();
     }
 
@@ -373,13 +392,17 @@ sub to_perl {
 ##############################################
 # push functions
 ##############################################
+sub delete_function {
+    my $sub = shift;
+    delete $Functions->{"$sub"};
+}
+
 sub push_function {
     my $self = shift;
     my $sub = shift;
     my $nargs = shift;
     if (!defined $nargs){ $nargs = -1 }
-    $SUB++;
-    $Functions->{$SUB} = sub {
+    $Functions->{"$sub"} = sub {
         my $top = $self->get_top();
         my $ret = 1;
         $self->perl_duk_safe_call(sub {
@@ -398,8 +421,7 @@ sub push_function {
         }, $top, 1);
         return $ret;
     };
-
-    $self->perl_push_function($Functions->{$SUB}, $nargs);
+    $self->perl_push_function($Functions->{"$sub"}, $nargs);
 }
 
 *push_c_function = \&push_function;
